// 道具メニュー制御オブジェクト
function ItemDialog() {

	// プレイヤーアイテム情報
	this._itemHolder = null;

	// 描画コンテキスト
	this._context = null;

	// 道具一覧ダイアログ
	this._listDlg = new SelectDialog();

	// 道具操作ダイアログ
	this._itemDlg = new SelectDialog();

	// メッセージウインドウ
	this._message = new MessageWindow();

	this._status;

	this._x = 0;
	this._y = 0;

	this._itemIndex = -1;
	this._item = null;

	this.resultItem = null;
	this.result = 0;

	this._visible = false;

	this.setup = function(holder, context) {

		this._itemHolder = holder;
		this._context = context;

		this._visible = false;

		this.setupListDlg();

	}

	this.setupListDlg = function() {

		var i;
		var list;

		list = new Array();

		list.push("戻る");

		for (i = 0;i < this._itemHolder.getItemNum();i++) {
			if (this._itemHolder.getItem(i).equip) {
				list.push('(E)' + this._itemHolder.getItem(i).name);
			} else {
				list.push(this._itemHolder.getItem(i).name);
			}
		}

		this._listDlg.setItems(list, this._context);
		this._listDlg.reset();

		this._listDlg.setCancelable(true);

		this._listDlg.setPos(this._x - this._listDlg.getWidth(), this._y - (this._listDlg.getHeight() / 2));

	}

	this.reset = function() {

		this._status = ItemDialog.STATUS_LIST_INIT;

		this._itemIndex = -1;
		this._item = null;

		this.resultItem = null;

	}

	this.setPos = function(x, y) {

		this._x = x;
		this._y = y;

	}

	this.process = function(key) {

		var list;

		switch (this._status) {

		// 道具一覧ダイアログ準備
		case ItemDialog.STATUS_LIST_INIT:

			this.setupListDlg();

			this._status = ItemDialog.STATUS_LIST;

			this._listDlg.show();

			break;

		// 道具一覧ダイアログ選択中
		case ItemDialog.STATUS_LIST:

			if (this._listDlg.process(key)) {

				switch (this._listDlg.getIndex()) {

				// キャンセル・戻る
				case -1:
				case 0:

					this._status = ItemDialog.STATUS_CANCELED;

					this._itemIndex = -1;
					this._item = null;

					this.resultItem = null;

					return true;

				//　選択
				default:

					// 選択結果を記録
					this._itemIndex = this._listDlg.getIndex() - 1;

					// 選択された道具
					this._item = this._itemHolder.getItem(this._itemIndex);

					// 道具操作ダイアログへ
					this._status = ItemDialog.STATUS_ITEM_INIT;

					break;

				}

			}

			break;

		// 道具操作ダイアログ準備
		case ItemDialog.STATUS_ITEM_INIT:

			// 選択肢配列を作成
			list = new Array();

			list.push("戻る");

			// 装備可能な道具なら装備を追加
			if (this._item.type != PlayerItem.ITEMTYPE_GOODS) {
				list.push("装備");
			}

			list.push("使う");
			list.push("捨てる");

			this._itemDlg.reset();

			this._itemDlg.setItems(list, this._context);
			this._itemDlg.setCancelable(true);

			this._status = ItemDialog.STATUS_ITEM;

			this._itemDlg.setPos(this._x, this._y - (this._itemDlg.getHeight() / 2));

			this._itemDlg.show();

			break;

		// 道具操作ダイアログ選択中
		case ItemDialog.STATUS_ITEM:

			if (this._itemDlg.process(key)) {

				switch (this._itemDlg.getIndex()) {

				case -1:
				case 0:

					this._status = ItemDialog.STATUS_LIST_INIT;

					this._itemDlg.hide();

					break;

				case 1:

					this.resultItem = this._item;

					if (this._item.type == PlayerItem.ITEMTYPE_GOODS) {
						this.result = ItemDialog.RESULT_USE;
					} else {
						this.result = ItemDialog.RESULT_EQUIP;
					}

					this._itemDlg.hide();

					return true;

				case 2:

					this.resultItem = this._item;

					if (this._item.type == PlayerItem.ITEMTYPE_GOODS) {
						this.result = ItemDialog.RESULT_DISPOSE;
					} else {
						this.result = ItemDialog.RESULT_USE;
					}

					this._itemDlg.hide();

					return true;

				case 3:

					this.resultItem = this._item;

					this.result = ItemDialog.RESULT_DISPOSE;

					this._itemDlg.clear();
					this._itemDlg.hide();

					return true;

				}

			}

			break;

		}

		return false;

	}

	this.show = function() {
		this._visible = true;
	}

	this.hide = function() {
		this._visible = false;
	}

	this.draw = function(context) {

		if (!this._visible) {
			return;
		}

		this._listDlg.draw(context);
		this._itemDlg.draw(context);

	}

}

ItemDialog.STATUS_NONE = 0;
ItemDialog.STATUS_LIST_INIT = 1;
ItemDialog.STATUS_LIST = 2;
ItemDialog.STATUS_ITEM_INIT = 3;
ItemDialog.STATUS_ITEM = 4;
ItemDialog.STATUS_CANCELED = 5;

ItemDialog.RESULT_EQUIP = 1;
ItemDialog.RESULT_USE = 2;
ItemDialog.RESULT_DISPOSE = 3;
